# Schmant build script for building HeliDB from an Eclipse workspace.
# + The working directory must be the HeliDB_resources/build directory.
# + Must have Xalan JAR:s (serializer.jar and xalan.jar) on classpath if 
#   noDocbook is not set.
#
# Optional properties:
#  version      - The version of HeliDB built. This will be inserted here and
#                 there throughout the documentation
#  noFindbugs   - Do not run Findbugs
#  noDocbook    - Do not build Docbook documentation
#  noJavadoc    - Do not build the Javadocs
#  linkValidation - Validate HTML links
#  buildThreads - Number of build threads for executors. Defaults to the number
#                 of CPU:s * 2.
#  dependencies - A Map<String, EntityView> with mappings from external
#                 dependency name to their EntityView:s (files or 
#                 directories). This provides a way of overriding the default
#                 dependencies in the resource project's lib directory.
#  docbookRoot  - Path to the Docbook schema distribution. Default is
#                 /usr/share/xml/docbook.
#
# Environment variables:
#  JAVA_HOME    - Point to a JDK 1.6+.
#  FINDBUGS_HOME - Point to Findbugs installation.

#
# CONSTANTS
#
XSL_TRANSFORMER = "org.apache.xalan.processor.TransformerFactoryImpl"

#
# FUNCTIONS
#

# Create a task that creates a HTML file/book from a Docbook XML document
def createDocbookToXHtmlTask():
	return XsltTF(). \
		setTransformerFactoryClassName(XSL_TRANSFORMER). \
		setParameter("img.src.path", ""). \
		setParameter("use.role.for.mediaobject", "1"). \
		setParameter("chunk.section.depth", "0")

# Get a TextReplaceTF that is used for post processing of documentation files.
def getPostProcessReplacementTF(ramTempDir):
	return ReplaceSourceFileTF(). \
		setLogHeader(None). \
		addTaskFactory( \
			ApiLinksTF(). \
			    setTempDirectory(ramTempDir). \
				setLinkClass("apilink"). \
				addApiLink(ApiLink("org.entityfs.", "http://entityfs.sourceforge.net/releases/current/api/index.html")). \
				addApiLink(ApiLink("org.helidb.", "../api/index.html?")). \
				addApiLink(ApiLink("java.", "http://java.sun.com/javase/6/docs/api/index.html")). \
				addApiLink(ApiLink("javax.", "http://java.sun.com/javase/6/docs/api/index.html")). \
				addApiLink(ApiLink("org.xml.", "http://java.sun.com/javase/6/docs/api/index.html"))). \
		addTaskFactory( \
			TextReplaceTF(). \
				addReplace("###VERSION###", version))

def createAddEncodingHeaderTF():
	# \\0 in the regular expression returns the zeroth capture group, i.e. the
	# whole matched string
	return TextReplaceTF(). \
		setLogHeader(None). \
		addReplace( \
			Pattern.compile("<\\s*head(\\p{Print}|\\s)*?>", Pattern.CASE_INSENSITIVE), \
			"\\0<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>")

#
# START HERE
#
 
# The target directory where we'll build the source and binary distributions.
# Keep this directory
targetDir = TempFileUtil.createTempDirectory("helidbbuild", None, True)

# This is where build artifacts will be put while building. Everything under
# this directory will be deleted when Schmant exits.
buildDir = TempFileUtil.createTempDirectory(targetDir)

# Directories for the source distribution (uses files from binary distribution too)
sdRoot = Directories.newDirectory(buildDir, "srcdist")
sdBuildDir = Directories.newDirectory(sdRoot, "build")
sdBuildLibDir = Directories.newDirectory(sdBuildDir, "lib")

# Directories for the binary distribution
bdRoot = Directories.newDirectory(buildDir, "bindist")
docDir = Directories.newDirectory(bdRoot, "doc")
bdLibDir = Directories.newDirectory(bdRoot, "lib")
bdOptlibDir = Directories.newDirectory(bdRoot, "optlib")

# Directories for temporary build files
tmpRoot = TempFileUtil.createTempDirectory(buildDir)
# Use this as the default temporary directory
TempFileUtil.setDefaultTempDirectory(tmpRoot)

tmpJarDir = Directories.newDirectory(tmpRoot, "jar")
tmpSrcDir = Directories.newDirectory(tmpRoot, "src")
tmpClassDir = Directories.newDirectory(tmpRoot, "classes")
tmpJUnitDir = Directories.newDirectory(tmpRoot, "junit4")
	
# A RAM file system for temporary files
ramTmpDir = SchmantFileSystems.createRamFileSystem()

# A read-only file system for the Eclipse workspace
wosFS = FSROFileSystemBuilder(). \
	setRoot(File("../..")). \
	setName("Eclipse Workspace"). \
	create()

# An EntityFilter that blocks Subversion .svn directories
noSubversionFilter = DirectoryFilter.FILTER.and(EntityNameFilter(".svn")).not()
	
# Get the workspace's root directory with all .svn directories hidden.
wosRoot = wosFS.getRootDirectory().newView(noSubversionFilter)
wosResourcesDir = Directories.getDirectory(wosRoot, "HeliDB_resources")
wosLibDir = Directories.getDirectory(wosResourcesDir, "lib")
wosOptlibDir = Directories.getDirectory(wosResourcesDir, "optlib")
wosBuildDir = Directories.getDirectory(wosResourcesDir, "build")
wosTestDataDir = Directories.getDirectory(wosResourcesDir, "testdata")
wosDocDir = Directories.getDirectory(wosRoot, "HeliDB_doc")

# Load helidb_build.properties
buildProperties = PropertiesUtil.loadFromFile(Directories.getFile(wosBuildDir, "helidb_build.properties"))

# Version information should be provided by the user running the script
version = props.getStringValue("version", "no_version_set")

# The Eclipse workspace
eWos = EclipseWorkspace(wosRoot)

from java.lang import Math
from java.lang import Runtime
from java.util.regex import Pattern

# The executor
te = TaskExecutor(). \
	setNumberOfThreads(props.getIntValue("buildThreads", Math.max(Runtime.getRuntime().availableProcessors(), 2))). \
	start()
try:
	javaProjects = ProjectFilterUtil.filter(eWos.getProjects(), ProjectTypeFilter(JavaProjectType.TYPE))
	
	# First, preprocess all the Java source files
	
	# This is a dependency object for the source copy tasks
	copySourceDep = CompoundTaskDependency()
	
	# A mapping project name -> preprocessed source directory
	projectSourceDirs = HashMap()

	# Copy all source directories to a temporary location
	
	itr = javaProjects.iterator()
	while itr.hasNext():
		proj = itr.next()
		projName = proj.getName()
		projSourceDir = Directories.newDirectory(tmpSrcDir, projName)
		projectSourceDirs[projSourceDir] = projName

		tct = TreeCopyTF(). \
			setLogHeader("Copying source code from project " + projName). \
			addSources(proj.getSourceDirectories()). \
			setTarget(projSourceDir).create()
		copySourceDep.add(tct)
		te.add(tct)
	
	# End while
		
	# This is a dependency object for all preprocessing tasks
	preProDep = CompoundTaskDependency()
	
	# Add the @injar Javadoc tag to source files for classes that will be
	# included in the partial jars
	jars = buildProperties.getStringArrayValue("jars")
	for jarName in jars:
		projs = buildProperties.getStringArrayValue("jar." + jarName + ".projects")

		for projName in projs:
			projSourceDir = Directories.getDirectory(tmpSrcDir, projName)
			
			proj = eWos.getProject(projName)

			# The regexp matches the class, enum or interface Javadoc comment
			# for public classes.
			# \\1 returns the first capture group, i.e. the class or interface
			# comment up to the terminating */
			# \\2 returns the second capture group, i.e. everything matched after */
			ijt = RecursiveActionTF(). \
				setLogHeader("Adding the @injar Javadoc tag to the source files of " + projName). \
				setSource(DirectoryAndFilter(projSourceDir, EFileFilter.FILTER)). \
				setTaskFactory( \
				    ReplaceSourceFileTF(). \
					    setTaskFactory( \
							TextReplaceTF(). \
								addReplace( \
									Pattern.compile("(/\\*\\*.*?)\\*/(\\s*public\\s+(abstract\\s+)?(final\\s+)?((class)|(interface)|(enum)))", Pattern.MULTILINE | Pattern.DOTALL), \
									"\\1* @injar helidb-" + jarName + "\n */\\2"))).create()
									
			te.add(ijt, copySourceDep)
			preProDep.add(ijt)
		
		# End for
	# End for
	
	buildWorkspace = JavaWorkspaceBuilderTF(). \
		setWorkspace(eWos). \
		setCopyNonJavaSourceFilesToTarget(True). \
		setProjectSpecificTargetDirectories(True). \
		addProjectSourceDirectories(projectSourceDirs). \
		setCompileTaskFactory( \
			Jdk6JavacTF(). \
				addArgument("-Xlint"). \
				setTargetVersion("5"). \
				setEncoding("UTF-8")). \
		setTaskExecutor(te). \
		setTarget(tmpClassDir).create()
	te.add(buildWorkspace, preProDep)
	
	te.waitFor()

finally:
	te.shutdown()

classDirs = Directories.listEntities(tmpClassDir)

te = TaskExecutor(). \
	setNumberOfThreads(props.getIntValue("buildThreads", Math.max(Runtime.getRuntime().availableProcessors(), 2))). \
	start()

try:
	# A dependency for all Jar tasks
	jarDep = CompoundTaskDependency()
	
	helidbJar = FutureFile(bdLibDir, "helidb-full-" + version + ".jar")
	# Build a jar with all non-test projects
	jt = JarTF(). \
	    setLevel(9). \
		setTarget(helidbJar). \
		addSources( \
			EntityFilters.filterCollection( \
				classDirs, \
				EntityNameGlobFilter("HeliDB_test_support*").not().and( \
					EntityNameFilter("HeliDB_resources_perftest").not().and( \
					EntityNameRegexpFilter(".*_test").not())))).create()
	jarDep.add(jt)
	te.add(jt)
		
	helidbJUnitJar = FutureFile(bdLibDir, "helidb-junit4-" + version + ".jar")
	# Build a jar with all test projects
	jt = JarTF(). \
		setLevel(9). \
		setTarget(helidbJUnitJar). \
		addSources( \
			EntityFilters.filterCollection( \
				classDirs, \
				EntityNameRegexpFilter(".*_test").or(
					EntityNameRegexpFilter("HeliDB_test_support")))).create()
	jarDep.add(jt)
	te.add(jt)
	
	if not props.containsKey("noFindbugs"):
		enableTaskPackage("net.findbugs")
			
		findbugsDir = Directories.newDirectory(docDir, "findbugs")
		
		# Schedule tasks for Findbugs-analyzing each Java project
		itr = ProjectFilterUtil.filter( \
			javaProjects, \
			ProjectNameGlobFilter("*_test").not().and( \
				ProjectNameFilter("HeliDB_resources_perftest").not()).and( \
				ProjectNameFilter("HeliDB_test_support").not())).iterator()

		while itr.hasNext():
			proj = itr.next()
			projName = proj.getName()
			
			if projName.endswith("_test"):
				excludeFile = "findbugs_exclude_for_test_project.xml"
			else:
				excludeFile = "findbugs_exclude.xml"
				
			te.add( \
				ExtFindbugsTF(). \
					setLogHeader("Running Findbugs on " + proj.toString()). \
					addAuxClasspathEntry(JdkUtil.getJdkFile("jre/lib/rt.jar")). \
					addAuxClasspathEntries(Directories.getAllFilesMatching(wosLibDir, "*.jar")). \
					addAuxClasspathEntry(helidbJar). \
					addAuxClasspathEntry(helidbJUnitJar). \
					addSourceCodeContainer(Directories.getDirectory(tmpSrcDir, projName)). \
					addSource(Directories.getDirectory(tmpClassDir, projName)). \
					setEffortLevel(FindbugsEffortLevel.MAX). \
					setFindbugsReportLevel(FindbugsReportLevel.LOW). \
					setFindbugsReportFormat(FindbugsReportFormat.HTML). \
					setExcludeFilterFile(Directories.getFile(wosBuildDir, excludeFile)). \
					setTarget(FutureFile(findbugsDir, projName + ".html")), \
				jarDep)
		
		# End while
	# End if

	# Copy build files to source distribution
	te.add( \
		TreeCopyTF(). \
			setLogHeader("Copying files from build directory to source distribution"). \
			addSource(wosBuildDir). \
			setTarget(sdBuildDir))
	
	# Create an helidb_perftest jar in build/lib
	te.add(
		JarTF(). \
			setLogHeader("Building Jar helidb_perftest.jar"). \
			setLevel(9). \
			setTarget(FutureFile(sdBuildLibDir, "helidb_perftest.jar")). \
			addSource(FutureDirectory(tmpClassDir, "HeliDB_resources_perftest")), \
			jarDep)
		
	# Copy test data files to source distribution
#//	te.add(
#//		TreeCopyTF().
#//			addSource(wosTestDataDir).
#//			setTarget(Directories.newDirectory(sdRoot, "testdata")));
			
	# Build smaller, partial jars
	jars = buildProperties.getStringArrayValue("jars")
	for jarName in jars:
		sourceDirs = []
		for projName in buildProperties.getStringArrayValue("jar." + jarName + ".projects"):
			p = eWos.getProject(projName)
			if p != None:
				sourceDirs.append(Directories.getDirectory(tmpClassDir, projName))
			else:
				warn("No project with name " + projName + ". Ignoring this when building " + jarName)
		# End for
		
		te.add(
			JarTF().
				setLogHeader("Building Jar " + jarName).
				setLevel(9).
				setTarget(FutureFile(bdLibDir, "helidb-" + jarName + "-" + version + ".jar")).
				addSources(sourceDirs),
			jarDep)
	# End for
		
	if not props.containsKey("noJavadoc"):
		javadocDir = Directories.newDirectory(tmpRoot, "api")
	
		sourceDirs = []
		
		# Add source directories for all non-test Java projects.
		allNonTestProjects = ProjectFilterUtil.filter( \
			javaProjects, \
			ProjectNameGlobFilter("*_test").or( \
			ProjectNameGlobFilter("*_test_support")).or( \
			ProjectNameGlobFilter("HeliDB_resources_*")).not())
			
		itr = allNonTestProjects.iterator()
		while itr.hasNext():
			sourceDirs.append(Directories.getDirectory(tmpSrcDir, itr.next().getName()))
		
		from java.util.logging import Level
		
		# A task for building the Javadocs (all non-test projects)
		te.add( \
			ExtJavadocTF().
				setLogHeader("Building Javadocs"). \
				setReportLevel(Level.WARNING). \
				addPackageName("org.helidb"). \
				setSubPackages(True). \
				addTag(
					JavadocTagDefinition.createTag("injar"). \
						addPlace(JavadocTagPlace.TYPES). \
						setTagText("In_jar:")). \
				addSources(sourceDirs). \
				addClasspathEntries(
					Directories.getAllFilesMatching(wosLibDir, "*.jar")). \
				addDecorator( \
					ExtStandardDocletDecorator(). \
						setTarget(javadocDir). \
						setAuthor(True). \
						addLink("http://java.sun.com/javase/6/docs/api/"). \
						addLink("http://entityfs.sourceforge.net/releases/current/api"). \
						setKeywords(True). \
						setDocTitle("HeliDB version " + version + " Javadocs"). \
						setWindowTitle("HeliDB version " + version + " Javadocs")),
			jarDep)
	# End if			
		
	if not props.containsKey("noDocbook"):
		if not props.containsKey("docbookRoot"):
			warn("The docbookRoot property is not set. Defaulting to /usr/share/xml/docbook")

		docbookRoot = FileSystems.getEntityForDirectory(File(props.getStringValue("docbookRoot", "/usr/share/xml/docbook")), True)
		
		# A catalog resolver that will be used for resolving external entities.
		# Instruct it to use Files instead of InputStreams for creating URI
		# Source:s
		cr = XmlCatalogResolver().setUseFilesInsteadOfStreamsForUris(True)
		
		# Add the Docbook DTD to the catalog
#		cr.addSystemId("http://www.oasis-open.org/docbook/xml/4.4/docbookx.dtd", Directories.getFile(wosBuildDir, RelativeLocation("docbook-4.4/docbookx.dtd")))
		
		# Need Saxon 6 TransformerFactory or the Xalan TransformerFactory as 
		# they seem to be the only TransformerFactories that can parse 
		# Docbook XSL:s.

		catalogBuildDep = CompoundTaskDependency()
	
		# Java cannot handle relative paths in XSL files
		# (see http://www.sagehill.net/docbookxsl/WriteCatalog.html )
		# We have to create a temporary XML catalog for the XSL entities
		t = RecursiveActionTF(). \
			setLogHeader("Adding Docbook XSL files to XML catalog"). \
			setSource(DirectoryAndFilter(docbookRoot, EFileFilter.FILTER)). \
			setTaskFactory( \
				AddUriToCatalogTF(). \
					setXmlCatalog(cr). \
					setBaseLocation(AbsoluteLocation("/xhtml"))).create()
		catalogBuildDep.add(t)
		te.add(t)
		
		t = RecursiveActionTF(). \
			setLogHeader("Adding Programmer's guide entities to XML catalog"). \
			addSource( \
				DirectoryAndFilter( \
					Directories.getDirectory(wosDocDir, "pg"), \
					EFileFilter.FILTER)). \
			setTaskFactory( \
				AddSystemIdToCatalogTF(). \
					setXmlCatalog(cr). \
					setLocationPrefix("http://pg/")).create()
		catalogBuildDep.add(t)
		te.add(t)
		
		# Dump the XML catalog
#		tmpt = ScriptTF(). \
#			setScript(CharSequenceReadableFile("java.lang.System.out.println(cr);")). \
#			putVariable("cr", cr).create()
#		te.add(tmpt, catalogBuildDep)
#		catalogBuildDep = tmpt
		
		ss = TemplateCompilerTF(). \
			setLogHeader("Compiling Docbook single HTML XSL stylesheet"). \
			setTransformerFactoryClassName(XSL_TRANSFORMER). \
			setSource(Directories.getFile(docbookRoot, RelativeLocation("xhtml/docbook.xsl"))). \
			setXslUriResolver(cr).create()
		te.add(ss, catalogBuildDep)

		cs = TemplateCompilerTF(). \
			setLogHeader("Compiling Docbook chunked HTML XSL stylesheet"). \
			setTransformerFactoryClassName(XSL_TRANSFORMER). \
			setSource(Directories.getFile(docbookRoot, RelativeLocation("xhtml/chunk.xsl"))). \
			setXslUriResolver(cr).create()
		te.add(cs, catalogBuildDep)
		
		wosPgDir = Directories.getDirectory(wosDocDir, "pg")

		parsePG = DomParseXmlTF(). \
			setLogHeader("Parsing Programmer's Guide XML"). \
			setNamespaceAware(True). \
			setEntityResolver(cr). \
			setSource(Directories.getFile(wosPgDir, "pg.xml")).create()		
		te.add(parsePG, catalogBuildDep)
		
		pgDir = Directories.newDirectory(docDir, "pg")
		mpPgToHtml = createDocbookToXHtmlTask(). \
			setLogHeader("Creating Programmer's Guide multi-page HTML book"). \
			setSource(parsePG). \
			setTemplates(cs). \
			setTarget(FutureFile(pgDir, "pg.html")). \
			setUriResolver(cr). \
			setParameter("html.stylesheet", "../helidb_doc.css"). \
			setParameter("base.dir", ECFileResolvableUtil.getFileObject(pgDir).getAbsolutePath() + File.separator).create()
		te.add(mpPgToHtml, [ parsePG, cs ])

		# This creates an empty pg.html file in the documentation directory. Delete it.
		ttt = DeleteTF(). \
			setSource(FutureFile(pgDir, "pg.html")).create()
		te.add(ttt, mpPgToHtml)
		
		# Postprocess the programmer's guide
		ugpp = RecursiveActionTF(). \
			setLogHeader("Post-processing the multi-page Programmer's Guide"). \
			setSource(DirectoryAndFilter(pgDir, EFileNameExtensionFilter("html"))). \
			setTaskFactory( \
				getPostProcessReplacementTF(ramTmpDir)).create()
		te.add(ugpp, ttt)

		te.add( \
			RecursiveActionTF(). \
				setLogHeader("Adding character encoding information to the multi-page Programmer's Guide files"). \
				setSource(DirectoryAndFilter(pgDir, EFileNameExtensionFilter("html"))). \
				setTaskFactory( \
					ReplaceSourceFileTF(). \
						setTaskFactory(createAddEncodingHeaderTF())), \
			ugpp)
		
		# Copy all PNG images to the programmer's guide
		te.add( \
			TreeCopyTF(). \
				setLogHeader("Copying images to the Programmer's Guide"). \
				setSource(wosPgDir.newView(EFileNameExtensionFilter("png"))). \
				setTarget(pgDir))

		spPgToHtml = createDocbookToXHtmlTask(). \
			setLogHeader("Creating Programmer's Guide single page HTML book"). \
			setSource(parsePG). \
			setTemplates(ss). \
			setTarget(FutureFile(targetDir, "pg-single.html")). \
			setUriResolver(cr). \
			setParameter("html.stylesheet", "../helidb_doc.css"). \
			setParameter("base.dir", ECFileResolvableUtil.getFileObject(targetDir).getAbsolutePath() + File.separator).create()
		# Let this depend on the multi page task since the docbook style sheets
		# + XSLT transformation is not a thread safe operation.
		te.add(spPgToHtml, [ parsePG, ss, mpPgToHtml ]);

		pgSingle = FutureFile(targetDir, "pg-single.html")
		
		# Postprocess the single page user's guide
		ugspp = getPostProcessReplacementTF(ramTmpDir). \
			setLogHeader("Post-processing the single page Programmer's Guide"). \
			setSource(pgSingle).create()
		te.add(ugspp, spPgToHtml)

		te.add( \
			ReplaceSourceFileTF(). \
				setLogHeader("Adding character encoding information to the single page Programmer's Guide file"). \
				setSource(pgSingle). \
				setTaskFactory( \
					createAddEncodingHeaderTF()), \
			ugspp)
	# End if
	
	binaryFileFilter = EFileNameExtensionFilter("png")
	wosDocDocDir = Directories.getDirectory(wosDocDir, "doc")
	
	# Edit and copy other documentation files to the binary distribution. Don't
	# copy any binary files
	t = RecursiveProcessTF(). \
		setLogHeader("Adding version information to documentation and copying to /doc"). \
		addSource( \
			DirectoryAndFilter( \
				wosDocDocDir, \
				EFileFilter.FILTER.and(binaryFileFilter.not()))). \
		setTarget(docDir). \
		setTaskFactory( \
			TextReplaceTF(). \
				addReplace("###VERSION###", version)).create()
	te.add(t)
	
	# Copy binary files to binary distribution
	te.add( \
		TreeCopyTF(). \
			setLogHeader("Copying binary documentation files to distribution"). \
			addSource( \
				DirectoryAndFilter( \
					wosDocDocDir, \
					EFileFilter.FILTER.and(binaryFileFilter))). \
			setTarget(docDir))

	# Convert the style sheet to have Windoz newlines. IE does not work
	# otherwise
	te.add( \
		ReplaceSourceFileTF(). \
			setLogHeader("Converting documentation style sheet to Windows newlines"). \
			setSource(FutureFile(docDir, "helidb_doc.css")). \
			setTaskFactory( \
				FormatCharacterFileTF(). \
					setEolStyle(EolStyle.WIN)), \
		t)
	
	te.add( \
		RecursiveProcessTF(). \
			setLogHeader("Adding version information to release files and copying to /"). \
			addSource( \
				DirectoryAndFilter( \
					Directories.getDirectory(wosDocDir, "misc"), \
					EFileFilter.FILTER)). \
			setTarget(bdRoot). \
			setTaskFactory( \
				TextReplaceTF(). \
					setLogHeader(None). \
					addReplace("###VERSION###", version)))
			
	# Wait for all tasks to be completed
	te.waitFor()

finally:
	te.shutdown()


# The executor for building archive files
te = TaskExecutor(). \
	setNumberOfThreads(props.getIntValue("buildThreads", Runtime.getRuntime().availableProcessors() * 2)). \
	start()
try:
	archiveRootDir = "/helidb-" + version
		
	enableTaskPackage("org.at4j")

	binDistSources = [ \
		EntityAndAbsoluteLocation(bdRoot, archiveRootDir), \
		EntityAndAbsoluteLocation(wosLibDir, archiveRootDir + "/lib")]
		
	if not props.containsKey("noJavadoc"):
		binDistSources.append(EntityAndAbsoluteLocation(javadocDir, archiveRootDir + "/doc/api"))
			
	# Build a binary Zip distribution. Set the schmant.sh file to be executable
	# on Unix. Use maximum compression.
	zipBinTargetFile = FutureFile(targetDir, "helidb-" + version + ".zip")
	te.add( \
		At4JZipTF(). \
	    	setLogHeader("Creating " + zipBinTargetFile.toString()). \
			setTarget(zipBinTargetFile). \
			setCompressionLevel(CompressionLevel.BEST). \
			addSources(binDistSources))

	# Build a binary distribution tar.bz2 archive.
	te.add( \
		TarTF(). \
			setLogHeader("Creating binary distribution Tar archive"). \
			setTarget( \
				BZip2NewWritableFileProxy( \
					FutureFile(targetDir, "helidb-" + version + ".tar.bz2"), \
					DontOverwriteAndThrowException.INSTANCE)).
		addSources(binDistSources))
			
	srcDistSources = []
	# Copy the binDistSources list
	srcDistSources[:0] = binDistSources
	srcDistSources.append(EntityAndAbsoluteLocation(sdRoot, archiveRootDir))

	# A source object for the Eclipse workspace. It excludes the .metadata and 
	# HeliDB_releases directories and all "bin" directories with a parent
	# containing an underscore in the name (that should be a project directory)
	srcDistSources.append( \
		EntityAndAbsoluteLocation( \
			wosRoot.newView( \
				EntityNameFilter("HeliDB_releases").or( \
					EntityNameFilter(".metadata")).or( \
						EntityNameFilter("bin").and(DirectoryFilter.FILTER).and( \
							ParentFilter(EntityNameGlobFilter("*_*")))).not()), archiveRootDir + "/src"))
	
	# Create the source distribution Zip archive. Use maximum compression.
	zipSrcTargetFile = FutureFile(targetDir, "helidb-src-" + version + ".zip")
	te.add( \
		At4JZipTF(). \
	    	setLogHeader("Creating " + zipSrcTargetFile.toString()). \
			setTarget(zipSrcTargetFile). \
			setCompressionLevel(CompressionLevel.BEST). \
			addSources(srcDistSources))
								
	# Build source distribution Tar archive
	te.add( \
		TarTF(). \
			setLogHeader("Creating source distribution Tar archive"). \
		setTarget( \
			BZip2NewWritableFileProxy( \
				FutureFile(targetDir, "helidb-src-" + version + ".tar.bz2"), \
				DontOverwriteAndThrowException.INSTANCE)). \
		addSources(srcDistSources))
			
	te.waitFor()
	
finally:
	te.shutdown()

if props.containsKey("linkValidation"):
	# Unzip the source and binary distributions in separate catalogs and run
	# link validation on their contents
	sdistdir = TempFileUtil.createTempDirectory(tmpRoot)
	ZipFiles.unzip(zipSrcTargetFile.getFile(), sdistdir)
	
	# Remove the src directory
	IteratorDeleter(Directories.getDirectory(sdistdir, RelativeLocation("helidb-" + version + "/src"))).deleteEntities()
	bdistdir = TempFileUtil.createTempDirectory(tmpRoot)
	ZipFiles.unzip(zipBinTargetFile.getFile(), bdistdir)
	
	# A collection containing valid links (so that the same link is not checked
	# several times).
	vlc = Collections.synchronizedSet(HashSet());
	
	# Create a new task executor
	te = TaskExecutor(). \
		setNumberOfThreads(props.getIntValue("buildThreads", Runtime.getRuntime().availableProcessors() * 4)). \
		start()
	try:
		# Validate links in HTML pages
		te.add( \
			RecursiveActionTF(). \
				setDisableHeaderLogging(False). \
				addSource(DirectoryAndFilter(sdistdir, EFileNameExtensionFilter("html").or(EFileNameExtensionFilter("xhtml")))). \
				addSource(DirectoryAndFilter(bdistdir, EFileNameExtensionFilter("html").or(EFileNameExtensionFilter("xhtml")))). \
				setTaskExecutor(te). \
				setTaskFactory( \
					HtmlLinkValidationTF(). \
						setValidLinkCollection(vlc). \
					    addJavadocPackage("org.helidb"). \
						addIgnorePattern("http://java.sun.com/javase/.*?/docs/api/.*")))
		te.waitFor()

	finally:
		te.shutdown()
